package be.iminds.ilabt.jfed.rspec.statistics;

import javafx.util.Pair;

import javax.annotation.Nonnull;
import java.util.*;

class RSpecStatisticsBuilder {
    private boolean countWarnings;
    private boolean countErrors;

    private int count;
    private int countTotal;
    private int countRaw;
    private int countIpv4;
    private int countOpenflow;
    private int countVm;
    private int countRawTotal;
    private int countVmTotal;
    private int countIpv4Total;
    private int countOpenflowTotal;
    private int skippedCount;

    private Date date;
    @Nonnull private Set<String> explicitSkippedHwTypes;
    @Nonnull private Set<String> implicitSkippedHwTypes;
    @Nonnull private Set<String> countedHwTypes;
    @Nonnull private Map<String, RSpecStatistics.CountPair> countPerHwType;

    /**
     * All counts are zero initially
     */
    public RSpecStatisticsBuilder() {
        this.explicitSkippedHwTypes = new HashSet<>();
        this.implicitSkippedHwTypes = new HashSet<>();
        this.countedHwTypes = new HashSet<>();
        this.countPerHwType = new HashMap<>();
    }

    public RSpecStatisticsBuilder setCountWarnings(boolean countWarnings) {
        this.countWarnings = countWarnings;
        return this;
    }

    public RSpecStatisticsBuilder setCountErrors(boolean countErrors) {
        this.countErrors = countErrors;
        return this;
    }


    public void addCountWarning(String msg) {
        //msg not yet stored
        this.countWarnings = true;
    }
    public void addCountError(String msg) {
        //msg not yet stored
        this.countErrors = true;
    }

    public RSpecStatisticsBuilder setCount(int count) {
        this.count = count;
        return this;
    }
    public RSpecStatisticsBuilder incCount(int extraCount) {
        this.count += extraCount;
        return this;
    }
    public RSpecStatisticsBuilder incCount() {
        this.count += 1;
        return this;
    }

    public RSpecStatisticsBuilder setCountTotal(int countTotal) {
        this.countTotal = countTotal;
        return this;
    }
    public RSpecStatisticsBuilder incCountTotal() {
        this.countTotal += 1;
        return this;
    }
    public RSpecStatisticsBuilder incCountTotal(int extraCountTotal) {
        this.countTotal += extraCountTotal;
        return this;
    }

    public RSpecStatisticsBuilder setCountRaw(int countRaw) {
        this.countRaw = countRaw;
        return this;
    }
    public RSpecStatisticsBuilder incCountRaw(int extraCountRaw) {
        this.countRaw += extraCountRaw;
        return this;
    }

    public RSpecStatisticsBuilder setCountIpv4(int countIpv4) {
        this.countIpv4 = countIpv4;
        return this;
    }
    public RSpecStatisticsBuilder incCountIpv4(int extraCountIpv4) {
        this.countIpv4 += extraCountIpv4;
        return this;
    }

    public RSpecStatisticsBuilder setCountOpenflow(int countOpenflow) {
        this.countOpenflow = countOpenflow;
        return this;
    }
    public RSpecStatisticsBuilder incCountOpenflow() {
        this.countOpenflow += 1;
        return this;
    }
    public RSpecStatisticsBuilder incCountOpenflow(int extraCuntOpenflow) {
        this.countOpenflow += extraCuntOpenflow;
        return this;
    }

    public RSpecStatisticsBuilder setCountVm(int countVm) {
        this.countVm = countVm;
        return this;
    }
    public RSpecStatisticsBuilder incCountVm(int extraCountVm) {
        this.countVm += extraCountVm;
        return this;
    }

    public RSpecStatisticsBuilder setCountRawTotal(int countRawTotal) {
        this.countRawTotal = countRawTotal;
        return this;
    }
    public RSpecStatisticsBuilder incCountRawTotal(int extraCountRawTotal) {
        this.countRawTotal += extraCountRawTotal;
        return this;
    }

    public RSpecStatisticsBuilder setCountVmTotal(int countVmTotal) {
        this.countVmTotal = countVmTotal;
        return this;
    }
    public RSpecStatisticsBuilder incCountVmTotal(int extraCountVmTotal) {
        this.countVmTotal += extraCountVmTotal;
        return this;
    }

    public RSpecStatisticsBuilder setCountIpv4Total(int countIpv4Total) {
        this.countIpv4Total = countIpv4Total;
        return this;
    }
    public RSpecStatisticsBuilder incCountIpv4Total(int extraCountIpv4Total) {
        this.countIpv4Total += extraCountIpv4Total;
        return this;
    }

    public RSpecStatisticsBuilder setCountOpenflowTotal(int countOpenflowTotal) {
        this.countOpenflowTotal = countOpenflowTotal;
        return this;
    }
    public RSpecStatisticsBuilder incCountOpenflowTotal(int extraCountOpenflowTotal) {
        this.countOpenflowTotal += extraCountOpenflowTotal;
        return this;
    }
    public RSpecStatisticsBuilder incCountOpenflowTotal() {
        this.countOpenflowTotal += 1;
        return this;
    }

    public RSpecStatisticsBuilder setSkippedCount(int skippedCount) {
        this.skippedCount = skippedCount;
        return this;
    }
    public RSpecStatisticsBuilder incSkippedCount() {
        this.skippedCount++;
        return this;
    }
    public RSpecStatisticsBuilder incSkippedCount(int extra) {
        this.skippedCount += extra;
        return this;
    }


    public RSpecStatisticsBuilder setDate(@Nonnull Date date) {
        this.date = date;
        return this;
    }


    public RSpecStatisticsBuilder setExplicitSkippedHwTypes(@Nonnull Set<String> explicitSkippedHwTypes) {
        this.explicitSkippedHwTypes = explicitSkippedHwTypes;
        return this;
    }
    public RSpecStatisticsBuilder addExplicitSkippedHwType(@Nonnull String explicitSkippedHwType) {
        this.explicitSkippedHwTypes.add(explicitSkippedHwType);
        return this;
    }
    public RSpecStatisticsBuilder addExplicitSkippedHwTypes(@Nonnull Collection<String> explicitSkippedHwTypes) {
        this.explicitSkippedHwTypes.addAll(explicitSkippedHwTypes);
        return this;
    }

    public RSpecStatisticsBuilder setImplicitSkippedHwTypes(@Nonnull Set<String> implicitSkippedHwTypes) {
        this.implicitSkippedHwTypes = implicitSkippedHwTypes;
        return this;
    }
    public RSpecStatisticsBuilder addImplicitSkippedHwType(@Nonnull String implicitSkippedHwType) {
        this.implicitSkippedHwTypes.add(implicitSkippedHwType);
        return this;
    }
    public RSpecStatisticsBuilder addImplicitSkippedHwTypes(@Nonnull Collection<String> implicitSkippedHwTypes) {
        this.implicitSkippedHwTypes.addAll(implicitSkippedHwTypes);
        return this;
    }

    public RSpecStatisticsBuilder setCountedHwTypes(@Nonnull Set<String> countedHwTypes) {
        this.countedHwTypes = countedHwTypes;
        return this;
    }
    public RSpecStatisticsBuilder addCountedHwType(@Nonnull String countedHwType) {
        this.countedHwTypes.add(countedHwType);
        return this;
    }
    public RSpecStatisticsBuilder addCountedHwTypes(@Nonnull Collection<String> countedHwTypes) {
        this.countedHwTypes.addAll(countedHwTypes);
        return this;
    }

    public RSpecStatisticsBuilder setCountPerHwType(@Nonnull Map<String, RSpecStatistics.CountPair> countPerHwType) {
        this.countPerHwType = countPerHwType;
        return this;
    }
    public RSpecStatisticsBuilder incTotalForHwType(@Nonnull String hwType, int extra) {
        countPerHwType.compute(hwType,
                (k, pair) -> (pair == null) ?
                        new RSpecStatistics.CountPair(extra, 0) :
                        pair.newWithIncTotal(extra));
        return this;
    }
    public RSpecStatisticsBuilder incAvailableForHwType(@Nonnull String hwType, int extra) {
        countPerHwType.compute(hwType,
                (k, pair) -> (pair == null) ?
                        new RSpecStatistics.CountPair(0, extra) :
                        pair.newWithIncAvailable(extra));
        return this;
    }
    public RSpecStatisticsBuilder incForHwType(@Nonnull String hwType, int extraAvailable, int extraTotal) {
        countPerHwType.compute(hwType,
                (k, pair) -> (pair == null) ?
                        new RSpecStatistics.CountPair(extraTotal, extraAvailable) :
                        pair.newWithInc(extraTotal, extraAvailable));
        return this;
    }

    public RSpecStatistics create() {
        return new RSpecStatistics(countWarnings, countErrors, count, countTotal, countRaw, countIpv4, countOpenflow,
                countVm, countRawTotal, countVmTotal, countIpv4Total, countOpenflowTotal, skippedCount, date,
                explicitSkippedHwTypes, implicitSkippedHwTypes, countedHwTypes, countPerHwType);
    }


    //getters
    public int getCount() { return count; }
    public int getCountTotal() { return countTotal; }
    public int getCountRaw() { return countRaw; }
    public int getCountIpv4() { return countIpv4; }
    public int getCountOpenflow() { return countOpenflow; }
    public int getCountVm() { return countVm; }
    public int getCountRawTotal() { return countRawTotal; }
    public int getCountVmTotal() { return countVmTotal; }
    public int getCountIpv4Total() { return countIpv4Total; }
    public int getCountOpenflowTotal() { return countOpenflowTotal; }
    public boolean isCountWarnings() { return countWarnings; }
    public boolean isCountErrors() { return countErrors; }
    public int getSkippedCount() { return skippedCount; }
    public Date getDate() { return date; }
    @Nonnull public Set<String> getExplicitSkippedHwTypes() { return Collections.unmodifiableSet(explicitSkippedHwTypes); }
    @Nonnull public Set<String> getImplicitSkippedHwTypes() { return Collections.unmodifiableSet(implicitSkippedHwTypes); }
    @Nonnull public Set<String> getCountedHwTypes() { return Collections.unmodifiableSet(countedHwTypes); }
    @Nonnull public Map<String, RSpecStatistics.CountPair> getCountPerHwType() { return Collections.unmodifiableMap(countPerHwType); }
}